@@ -1,19 +1,24 @@
-from carrot.backends.base import BaseBackend, BaseMessage
-from anyjson import serialize, deserialize
-from ghettoq.backends import Connection
-from itertools import count
-from ghettoq.messaging import Empty as QueueEmpty
-from django.utils.datastructures import SortedDict
-from carrot.utils import gen_unique_id
-import sys
-import time
 import atexit
-import threading
+import socket
+import sys
+
+from itertools import count
+from django.utils.datastructures import SortedDict
+from time import time, sleep
+
+from anyjson import serialize, deserialize
+from carrot.backends.base import BaseBackend, BaseMessage
+from carrot.utils import gen_unique_id
+
+from ghettoq.backends import Connection
+from ghettoq.messaging import Empty as QueueEmpty
 
 
 class QualityOfService(object):
 
-    def __init__(self, resource, prefetch_count=None, interval=None):
+    def __init__(self, backend, resource, prefetch_count=None,
+            interval=None):
+        self.backend = backend
         self.resource = resource
         self.prefetch_count = prefetch_count
         self.interval = interval
@@ -31,8 +36,13 @@ class QualityOfService(object):
         self._delivered.pop(delivery_tag, None)
 
     def restore_unacked(self):
-        for message, queue_name in self._delivered.items():
-            self.resource.put(queue_name, message)
+        for delivery_tag, delivered in self._delivered.items():
+            message, queue_name = delivered
+            send = self.backend.prepare_message(message.body, 2, 10,
+                                                message.content_type,
+                                                message.content_encoding)
+            send["destination"] = queue_name
+            self.resource.put(queue_name, serialize(send))
         self._delivered = SortedDict()
 
     def requeue(self, delivery_tag):
@@ -73,6 +83,7 @@ class Message(BaseMessage):
         raise NotImplementedError(
             "The GhettoQ backend does not implement basic.reject")
 
+
 class MultiBackend(BaseBackend):
     Message = Message
     default_port = None
@@ -80,9 +91,8 @@ class MultiBackend(BaseBackend):
     interval = 1
     polling = True
     _prefetch_count = None
-    _t = threading.local()
-    _t.consumers = {}
-    _t.callbacks = {}
+    _consumers = {}
+    _callbacks = {}
 
     def __init__(self, connection, **kwargs):
         if not self.type:
@@ -111,28 +121,35 @@ class MultiBackend(BaseBackend):
     def queue_purge(self, queue, **kwargs):
         return self.channel.Queue(queue).purge()
 
-    def _poll(self, resource):
+    def _poll(self, resource, timeout=None):
+        time_elapsed = 0.0
         while True:
+            time_start = time()
             if self.qos_manager.can_consume():
                 try:
                     return resource.get()
                 except QueueEmpty:
                     pass
-            self.polling and time.sleep(self.interval)
+            time_elapsed += (time() - time_start)
+            if timeout and time_elapsed >= timeout:
+                raise socket.timeout("Operation timed out.")
+            if self.polling:
+                sleep(self.interval)
+                time_elapsed += self.interval
 
     def declare_consumer(self, queue, no_ack, callback, consumer_tag,
                          **kwargs):
-        self._t.consumers[consumer_tag] = queue
-        self._t.callbacks[queue] = callback
+        self._consumers[consumer_tag] = queue
+        self._callbacks[queue] = callback
 
     def drain_events(self, timeout=None):
-        queueset = self.channel.QueueSet(self._t.consumers.values())
-        payload, queue = self._poll(queueset)
+        queueset = self.channel.QueueSet(self._consumers.values())
+        payload, queue = self._poll(queueset, timeout=timeout)
 
-        if not queue or queue not in self._t.callbacks:
+        if not queue or queue not in self._callbacks:
             return
 
-        self._t.callbacks[queue](payload)
+        self._callbacks[queue](payload)
 
     def consume(self, limit=None):
         for total_message_count in count():
@@ -176,16 +193,17 @@ class MultiBackend(BaseBackend):
 
     def publish(self, message, exchange, routing_key, **kwargs):
         message["destination"] = exchange
-        self.channel.Queue(exchange).put(serialize(message))
+        self.channel.Queue(exchange).put(serialize(message),
+                                         priority=message["priority"])
 
     def cancel(self, consumer_tag):
         if not self._channel:
             return
-        queue = self._t.consumers.pop(consumer_tag, None)
-        self._t.callbacks.pop(queue, None)
+        queue = self._consumers.pop(consumer_tag, None)
+        self._callbacks.pop(queue, None)
 
     def close(self):
-        for consumer_tag in self._t.consumers.keys():
+        for consumer_tag in self._consumers.keys():
             self.cancel(consumer_tag)
         if self._channel:
             self._channel.close()
@@ -205,7 +223,7 @@ class MultiBackend(BaseBackend):
     @property
     def qos_manager(self):
         if self._qos_manager is None:
-            self._qos_manager = QualityOfService(self.channel)
+            self._qos_manager = QualityOfService(self, self.channel)
 
         # Update prefetch count / interval
         self._qos_manager.prefetch_count = self._prefetch_count
@@ -225,3 +243,8 @@ class Database(MultiBackend):
 
 class MongoDB(MultiBackend):
     type = "mongodb"
+
+
+class Beanstalk(MultiBackend):
+    type = "Beanstalk"
+    polling = False
